/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.FTP;
import java.io.File;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import sos.configuration.SOSConfiguration;
import sos.util.SOSStandardLogger;

import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.SOSOptionBoolean;
import com.sos.JSHelper.Options.SOSOptionFolderName;
import com.sos.JSHelper.Options.SOSOptionInteger;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.io.Files.JSFile;
import com.sos.VirtualFileSystem.Options.SOSConnection2Options;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsAlternate;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsSuperClass;

/**
* \class SOSFTPOptions 
* 
* \brief SOSFTPOptions - 
* 
* \details
*
* \section SOSFTPOptions.java_intro_sec Introduction
*
* \section SOSFTPOptions.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author KB
* @version $Id$16.05.2010
* \see reference
*
* Created on 26.07.2010 20:52:28
 */
/**
 * @author KB
 *
 */
public class SOSFTPOptions extends SOSFtpOptionsSuperClass {
	/**
	 * 
	 */
	private static final long	serialVersionUID			= 1L;
	private final String		conClassName				= "SOSFTPOptions";
	private Logger				logger						= Logger.getLogger(SOSFTPOptions.class);
	private Properties			propSOSFtpEnvironmentVars	= null;
	private Properties			schedulerParams				= null;
	private boolean				zeroByteFiles				= false;
	private boolean				zeroByteFilesStrict			= false;
	private boolean				zeroByteFilesRelaxed		= false;
	private boolean				flgCheckMandatoryDone		= false;
	// Map<String, String> variables = null;
	private Properties			propAllEnvironmentVariables	= null;
	public SOSOptionString		SecurityHashType			= new SOSOptionString(this, conClassName + ".security_hash_type", // HashMap-Key
																	"The Type of the security hash, e.g. MD5", // Titel
																	"MD5", // InitValue
																	"MD5", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionBoolean		DecompressAfterTransfer		= new SOSOptionBoolean(this, conClassName + ".Decompress_After_Transfer", // HashMap-Key
																	"Decompress zipped-files after transfer", // Titel
																	"false", // InitValue
																	"false", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionBoolean		ConcurrentTransfer			= new SOSOptionBoolean(this, conClassName + ".Concurrent_Transfer", // HashMap-Key
																	"Process transfers simultaneously", // Titel
																	"false", // InitValue
																	"false", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionBoolean		CheckSecurityHash			= new SOSOptionBoolean(this, conClassName + ".Check_Security_Hash", // HashMap-Key
																	"Decompress zipped-files after transfer", // Titel
																	"false", // InitValue
																	"false", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionInteger		MaxConcurrentTransfers		= new SOSOptionInteger(this, conClassName + ".Max_Concurrent_Transfers", // HashMap-Key
																	"Maximum Numbers of parallel transfers", // Titel
																	"5", // InitValue
																	"1", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionBoolean		CreateSecurityHashFile		= new SOSOptionBoolean(this, conClassName + ".create_security_hash_file", // HashMap-Key
																	"This parameter specifies whether the content of a", // Titel
																	"false", // InitValue
																	"false", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionBoolean		CreateSecurityHash			= new SOSOptionBoolean(this, conClassName + ".create_security_hash", // HashMap-Key
																	"This parameter specifies whether the content of a", // Titel
																	"true", // InitValue
																	"false", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionFolderName	SourceDir					= new SOSOptionFolderName(this, conClassName + ".source_dir", // HashMap-Key
																	"local_dir Local directory into which or from which", // Titel
																	"", // InitValue
																	"", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionFolderName	TargetDir					= new SOSOptionFolderName(this, conClassName + ".target_dir", // HashMap-Key
																	"target_dir directory into which or from which", // Titel
																	"", // InitValue
																	"", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionInteger		BufferSize					= new SOSOptionInteger(this, conClassName + ".buffer_Size", // HashMap-Key
																	"This parameter specifies the interval in seconds", // Titel
																	"64000", // InitValue
																	"4096", // DefaultValue
																	false // isMandatory
															);

	public SOSFTPOptions() {
		//
	}
	private SOSConnection2Options	objConnectionOptions	= null;

	/**
	 * \brief SOSFTPOptions
	 *
	 * \details
	 *
	 * @param pobjListener
	 */
	public SOSFTPOptions(JSListener pobjListener) {
		super(pobjListener);
	}

	/**
	 * \brief SOSFTPOptions
	 *
	 * \details
	 *
	 * @param JSSettings
	 * @throws Exception
	 */
	public SOSFTPOptions(HashMap<String, String> JSSettings) throws Exception {
		super(JSSettings);
		setChildClasses(JSSettings);
	}

	private void setChildClasses(HashMap<String, String> JSSettings) throws Exception {
		objConnectionOptions = new SOSConnection2Options(JSSettings);
	}

	/**
	 * 
	 * \brief CheckMandatory
	 *  
	 * \details
	 *
	 * \return 
	 * @throws JSExceptionMandatoryOptionMissing, Exception 
	 *
	 */
	@Override
	public void CheckMandatory() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::CheckMandatory";
		if (this.flgCheckMandatoryDone == true) {
			return;
		}
		this.operation.CheckMandatory();
		String localDir = this.local_dir.Value();
		if (isEmpty(localDir) == true) {
			String strPathName = this.file_path.Value();
			if (isNotEmpty(strPathName) == true) {
				JSFile fleT = new JSFile(strPathName);
				// fn.substring(0, fn.lastIndexOf(java.io.File.pathSeparatorChar));
				String strParent = fleT.getParent();
				if (strParent != null && strParent.equalsIgnoreCase(strPathName) == false) {
					this.local_dir.Value(fleT.getParent());
				}
			}
		}
		localDir = this.local_dir.Value();
		if (isEmpty(localDir) == true) {
			this.local_dir.Value(this.SourceDir.Value());
			localDir = this.local_dir.Value();
		}
		super.CheckMandatory();
		// TODO in die Options-Klasse, falls nicht schon drin ist ....
		if (localDir.startsWith("\\\\")) {
			while (localDir.indexOf("\\") != -1) {
				localDir = localDir.replace('\\', '/');
			}
		}
		this.local_dir.Value(localDir);
		// TODO in die Options-Klasse, falls nicht schon drin ist ....
		if (localDir.startsWith("file://")) {
			if (new File(createURI(localDir)).exists() == false) {
				throw new JobSchedulerException("SOSMDX-E-0000: local directory does not exist or is not accessible: " + localDir);
			}
		}
		if (replacing.IsNotEmpty() && replacement.IsEmpty()) {
			throw new JobSchedulerException("SOSMDX-E-0000: parameter is missing for specified parameter [replacing]: [replacement]");
		}
		if (replacing.IsEmpty() && replacement.IsNotEmpty()) {
			throw new JobSchedulerException("SOSMDX-E-0000: parameter is missing for specified parameter [replacement]: [replacing]");
		}
		if (this.append_files.value() == true) {
			if (compress_files.value() == true) {
				throw new JobSchedulerException(
						"SOSMDX-E-0000: unsupported parameter settings [append_files, compress_files]: cannot append to compressed files");
			}
			if (this.protocol.equalsIgnoreCase("ftp")) {
				// oke
			}
			else {
				throw new JobSchedulerException("SOSMDX-E-0000: append not supported by requested protocol '" + this.protocol.Value() + "'");
			}
		}
		// String strProt = this.protocol.Value();
		// if (strProt.equalsIgnoreCase("ftp")) {
		// this.host.CheckMandatory();
		// this.user.CheckMandatory();
		// this.host.CheckMandatory();
		// this.password.CheckMandatory();
		// }
		// else {
		// if (strProt.equalsIgnoreCase("sftp")) {
		// this.host.CheckMandatory();
		// if (BufferSize.value() > 32000) {
		// BufferSize.value(32000);
		// }
		// if (this.ssh_auth_file.IsNotEmpty()) {
		// }
		// else {
		// // if (this.ssh_auth_method.isPublicKey())
		// // throw new JobSchedulerException(
		// // "SOSMDX-E-0000: publickey specified, but no authentication filename was specified as parameter 'ssh_auth_file'");
		// }
		// }
		// }
		if (append_files.value() == true) {
			if (isAtomicTransfer()) {
				throw new JobSchedulerException("SOSMDX-E-0000: Append-mode not possible with atomic-transfer.");
			}
			if (this.compress_files.value() == true) {
				throw new JobSchedulerException("SOSMDX-E-0000: Append-mode not possible with compress-files.");
			}
		}
		if (this.file_path.IsNotEmpty()) {
			if (this.file_spec.IsNotEmpty()) {
				this.file_path.Value("");
				// throw new
				// JobSchedulerException(String.format("SOSMDX-E-0000: 'file_path=%1$s' and 'file_spec=%2$s' specified, but only one of them is allowed",file_path.Value(),
				// file_spec.Value()));
				logger.info(String.format(
						"SOSMDX-I-0000: 'file_path=%1$s' and 'file_spec=%2$s' specified, but only one of them is allowed. file_path ignored.",
						file_path.Value(), file_spec.Value()));
			}
			if (this.RecurseSubFolders.value() == true) {
				throw new JobSchedulerException(String.format("SOSMDX-E-0000: 'file_path=%1$s' and 'recursiv' specified, but only one of them is allowed",
						file_path.Value()));
			}
		}
		if (this.local_dir.IsEmpty() && this.file_spec.IsNotEmpty()) {
			throw new JobSchedulerException("SOSMDX-E-0000: 'file_spec' specified, but no value for 'local_dir' given (local_dir is empty).");
		}
		if (this.zero_byte_transfer.String2Bool() == true) {
			TransferZeroByteFiles(true);
			setZeroByteFilesStrict(false);
		}
		else {
			if (this.zero_byte_transfer.equalsIgnoreCase("strict")) {
				TransferZeroByteFiles(false);
				setZeroByteFilesStrict(true);
			}
			else {
				if (this.zero_byte_transfer.equalsIgnoreCase("relaxed")) {
					TransferZeroByteFiles(false);
					setZeroByteFilesStrict(false);
					setZeroByteFilesRelaxed(true);
				}
				else {
					TransferZeroByteFiles(false);
					setZeroByteFilesStrict(false);
				}
			}
		}
		getDataSourceType();
		getDataTargetType();
		if (this.CheckNotProcessedOptions.value() == true) {
			this.CheckNotProcessedOptions();
		}
		flgCheckMandatoryDone = true;
	} // private void CheckMandatory

	/**
	 * 
	 * \brief isAtomicTransfer
	 * 
	 * \details
	 *
	 * \return boolean
	 *
	 * @return
	 */
	public boolean isAtomicTransfer() {
		boolean flgIsAtomicTransfer = atomic_prefix.IsNotEmpty() || atomic_suffix.IsNotEmpty();
		return flgIsAtomicTransfer;
	}

	/**
	 * 
	 * \brief getEnvVars
	 * 
	 * \details
	 *
	 * \return Properties
	 *
	 * @return
	 * @throws Exception
	 */
	private Properties getEnvVars() throws Exception {
		// TODO raus hier. die Routine ist zu generell und mu in die Shell-Klasse
		try {
			propSOSFtpEnvironmentVars = new Properties();
			// TODO SchedulerParams hat hier nichts zu suchen. Die mssen "von oben" ber den JS-Adapter in die Klassen gereicht werden
			schedulerParams = new Properties();
			// TODO this *must* be a static global constant
			String conSchedulerEnvVarPrefix = "scheduler_param_";
			// TODO ber einen Parameter vorgeben lassen
			String conSOSFtpEnvVarPrefix = "sosftp_";
			int intSOSFtpEnvVarPrefixLen = conSOSFtpEnvVarPrefix.length();
			Map<String, String> objM = System.getenv();
			propAllEnvironmentVariables = new Properties();
			propAllEnvironmentVariables.putAll(objM);
			for (Object k : propAllEnvironmentVariables.keySet()) {
				String key = (String) k;
				String value = (String) propAllEnvironmentVariables.get(k);
				//				logger.debug("getEnvVars() - " + key + "=" + value); //$NON-NLS-1$ //$NON-NLS-2$
				if (key.startsWith(conSOSFtpEnvVarPrefix)) {
					key = key.substring(intSOSFtpEnvVarPrefixLen);
					propSOSFtpEnvironmentVars.setProperty(key, value);
					continue;
				}
				if (key.startsWith("current_pid") || key.startsWith("ppid")) {
					propSOSFtpEnvironmentVars.setProperty(key, value);
					continue;
				}
				// TODO obsolet. must come from the JSAdapter as a Property-Collection
				if (key.indexOf(conSchedulerEnvVarPrefix) > -1) {
					schedulerParams.setProperty(key.substring(conSchedulerEnvVarPrefix.length()), value);
					continue;
				}
			}
			return propSOSFtpEnvironmentVars;
		}
		catch (Exception e) {
			String strM = "error occurred reading environment: " + e.toString();
			logger.error(strM, e);
			throw new JobSchedulerException(strM);
		}
	}

	/**
	 * 
	 * \brief ReadSettingsFile
	 * 
	 * \details
	 * The Name of the Settings-File and the Name of the Profile (Section) has to be defined by the Options
	 * "settings" and "profile" before entering this method.
	 * \return void
	 *
	 */
	// TODO move 'ReadSettingsFile' to JSOptionClass
	public void ReadSettingsFile() {
		// TODO use the virtual file-system to allow access to diffent types of setting-stores
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::ReadSettingsFile";
		this.settings.CheckMandatory();
		this.profile.CheckMandatory();
		try {
			getEnvVars();// TODO replace SOSConfigurations by a "ini-File-only" approach, to avoid dependencies on job-scheduler
			SOSConfiguration objConf = new SOSConfiguration(settings.Value(), "globals", new SOSStandardLogger(0));
			Properties objGlobals = objConf.getParameterAsProperties();
			objConf = new SOSConfiguration(settings.Value(), profile.Value(), new SOSStandardLogger(0));
			if (objConf.getParameterAsProperties().size() <= 0) {
				String strM = String.format("SOSMDX-E-0000: Profile '%1$s' not found or empty in file '%2$s'", profile.Value(), settings.Value());
				logger.error(strM, new JobSchedulerException(strM));
				throw new JobSchedulerException(strM);
			}
			Properties objP = new Properties();
			objP.putAll(objGlobals);
			objP.putAll(objConf.getParameterAsProperties());
			while (true) {
				Properties objIncludes = new Properties();
				boolean flgIncludeFound = false;
				for (Object k : objP.keySet()) {
					String strKey = (String) k;
					String strValue = (String) objP.get(k);
					logger.debug("ReadSettingsFile() - this will be translated into log4j"); //$NON-NLS-1$
					logger.debug("ReadSettingsFile() - Property. Key = " + strKey + ", val = " + strValue); //$NON-NLS-1$ //$NON-NLS-2$
					if (isIncludeDirective(strKey)) {
						String[] strValues = strValue.split(",");
						for (String strV : strValues) {
							SOSConfiguration config_ = new SOSConfiguration(settings.Value(), strV, new SOSStandardLogger(0));
							if (config_.getParameterAsProperties().size() <= 0) {
								String strM = String.format("SOSMDX-E-0000: Include '%1$s' not found or empty in file '%2$s'", strV, settings.Value());
								logger.error(strM);
								throw new JobSchedulerException(strM);
							}
							objIncludes.putAll(config_.getParameterAsProperties());
						}
						flgIncludeFound = true;
					}
					else {
						if (isIniComment(strKey) == false) {
							// TODO check duplicate Entry but take into account, that includes are in effect
							objIncludes.put(strKey, strValue);
						}
					}
				} // for
				if (objIncludes.size() > 0 && flgIncludeFound == true) {
					objP = new Properties();
					objP.putAll(objIncludes);
				}
				else {
					break;
				}
			}
			HashMap<String, String> map = new HashMap<String, String>();
			for (Object k : objP.keySet()) {
				String strKey = (String) k;
				String strValue = (String) objP.get(k);
				if (hasVariableToSubstiture(strValue) == true) {
					logger.debug("ReadSettingsFile() - Key = " + strKey + ", value = " + strValue); //$NON-NLS-1$ //$NON-NLS-2$
					strValue = SubstituteVariables(strValue, objP, "${", "}");
					strValue = SubstituteVariables(strValue, objP, "%{", "}");
					strValue = SubstituteVariables(strValue, objP, "%", "%");
					strValue = SubstituteVariables(strValue, propSOSFtpEnvironmentVars, "${", "}");
					strValue = SubstituteVariables(strValue, propSOSFtpEnvironmentVars, "%{", "}");
					strValue = SubstituteVariables(strValue, propSOSFtpEnvironmentVars, "%", "%");
					strValue = SubstituteVariables(strValue, propAllEnvironmentVariables, "${", "}");
					strValue = SubstituteVariables(strValue, propAllEnvironmentVariables, "%{", "}");
					strValue = SubstituteVariables(strValue, propAllEnvironmentVariables, "%", "%");
					// TODO wrong place: has to come from the JS-Adapter as properties
					strValue = SubstituteVariables(strValue, schedulerParams, "${", "}");
					strValue = SubstituteVariables(strValue, schedulerParams, "%{", "}");
					strValue = SubstituteVariables(strValue, schedulerParams, "%", "%");
					if (hasVariableToSubstiture(strValue)) {
						logger.warn(String.format("SOSMDX-W-0000: possible wrong variable-value specified in '%1$s' of key '%2$s'. not substituted.", strValue,
								strKey));
					}
				}
				map.put(strKey, strValue);
			}
			this.setAllOptions(map);
			setChildClasses(map);
		}
		catch (Exception e) {
			logger.error("ReadSettingsFile()", e); //$NON-NLS-1$
			throw new JobSchedulerException(e.getMessage(), e);
		}
	} // private void ReadSettingsFile

	private boolean hasVariableToSubstiture(final String pstrValue) {
		boolean flgResult = false;
		if (pstrValue.indexOf("$") != -1 || pstrValue.indexOf("%") != -1) {
			flgResult = true;
		}
		return flgResult;
	}

	private boolean isIniComment(final String pstrText) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isIniComment";
		return (pstrText.trim().startsWith(";"));
	} // private boolean isIniComment

	private boolean isIncludeDirective(final String pstrText) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isIniComment";
		return (pstrText.toLowerCase().startsWith("include"));
	} // private boolean isIniComment

	public String SubstituteVariables(String txt, Properties prop, String startPrefix, String endPrefix) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::SubstituteVariables";
		try {
			for (Object k : prop.keySet()) {
				String key = (String) k;
				String strValue = (String) prop.get(key);
				String strSearchFor = startPrefix + key + endPrefix;
				// logger.debug(String.format("search for  '%1$s' with value '%2$s'", strSearchFor, strValue));
				int pos1 = -1;
				int pos2 = 0;
				while (true) {
					pos1 = txt.indexOf(strSearchFor, pos2);
					if (pos1 == -1) {
						break;
					}
					logger.debug(String.format("found '%1$s'", strSearchFor));
					int intEscaped = txt.indexOf("\\" + strSearchFor);
					if (intEscaped > -1 && intEscaped == pos1 - 1)
						pos1 = -1;
					pos2 = pos1 + strSearchFor.length();
					if (pos1 > -1 && pos2 > pos1) {
						txt = txt.substring(0, pos1) + strValue + txt.substring(pos2);
						logger.debug(String.format("new Text = '%1$s'", txt));
					}
				} // while
			} // for
			return txt;
		}
		catch (Exception e) {
			throw new JobSchedulerException("can not substitute parameters in: " + txt, e);
		}
	}

	// TODO in die Klasse JSFile implementieren
	protected URI createURI(String fileName) {
		URI uri = null;
		try {
			uri = new URI(fileName);
		}
		catch (Exception e) {
			try {
				File f = new File(fileName);
				String path = f.getCanonicalPath();
				if (fileName.startsWith("/")) {
					path = fileName;
				}
				String fs = System.getProperty("file.separator");
				if (fs.length() == 1) {
					char sep = fs.charAt(0);
					if (sep != '/')
						path = path.replace(sep, '/');
					if (path.charAt(0) != '/')
						path = '/' + path;
				}
				if (!path.startsWith("file://")) {
					path = "file://" + path;
				}
				uri = new URI(path);
			}
			catch (Exception ex) {
				throw (new JobSchedulerException("SOSMDX-E-0000: error in createURI(): " + e.getMessage()));
			}
		}
		return uri;
	}

	/**
	 * 
	 * \brief OneOrMoreSingleFilesSpecified
	 * 
	 * \details
	 * Returns true, if one or more file-/path-names explicit specified
	 * (instead of path or RegExp).
	 * 
	 * \return boolean
	 *
	 * @return
	 */
	public boolean OneOrMoreSingleFilesSpecified() {
		boolean flgisFilePath = this.file_path.IsNotEmpty();
		return flgisFilePath;
	}

	/**
	 * 
	 * \brief getDataTargetType
	 * 
	 * \details
	 *
	 * \return String
	 *
	 * @return
	 */
	public String getDataTargetType() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getDataTargetType";
		String strDataTargetType = "";
		if (this.operation.Value().equalsIgnoreCase("send")) {
			strDataTargetType = this.protocol.Value();
			if (strDataTargetType.length() <= 0) {
				strDataTargetType = "local";
			}
			this.SourceDir.Value(this.local_dir.Value());
			this.TargetDir.Value(this.remote_dir.Value());
			SOSConnection2OptionsAlternate objT = this.getConnectionOptions().Target();
			objT.host.Value(this.host.Value());
			objT.port.value(this.port.value());
			objT.protocol.Value(this.protocol.Value());
			objT.passive_mode.Value(this.passive_mode.Value());
			SOSConnection2OptionsSuperClass objAlt = objT.Alternatives();
			objAlt.host.Value(this.alternative_host.Value());
			objAlt.port.value(this.alternative_port.value());
			objAlt.protocol.Value(this.protocol.Value());
			objT.passive_mode.Value(this.alternative_passive_mode.Value());
		}
		else {
			if (this.operation.Value().equalsIgnoreCase("receive")) {
				strDataTargetType = "local";
				this.SourceDir.Value(this.remote_dir.Value());
				this.TargetDir.Value(this.local_dir.Value());
			}
			else {
				strDataTargetType = this.getConnectionOptions().Target().protocol.Value();
				if (strDataTargetType.length() <= 0) {
					strDataTargetType = "local";
				}
				this.SourceDir.Value(this.local_dir.Value());
				this.TargetDir.Value(this.remote_dir.Value());
			}
		}
		logger.debug(String.format("DataTargetType = %1$s", strDataTargetType));
		return strDataTargetType;
	} // private String getDataTargetType

	public String getDataSourceType() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getDataSourceType";
		String strDataSourceType = "";
		if (this.operation.Value().equalsIgnoreCase("send")) {
			strDataSourceType = "local";
			changeDirValues();
			// if (this.getConnectionOptions().Source().protocol.IsNotEmpty()) {
			// strDataSourceType = this.getConnectionOptions().Source().protocol.Value();
			// }
			SOSConnection2OptionsAlternate objT = this.getConnectionOptions().Source();
			objT.host.Value("localhost");
			objT.port.value(0);
			objT.protocol.Value(strDataSourceType);
			objT = this.getConnectionOptions().Target();
			objT.host = this.host;
			objT.port = this.port;
			objT.protocol = this.protocol;
			objT.user = this.user;
			objT.password = this.password;
			objT.ssh_auth_file = this.ssh_auth_file;
			objT.ssh_auth_method = this.ssh_auth_method;
			objT.passive_mode = this.passive_mode;
			SOSConnection2OptionsSuperClass objAlt = objT.Alternatives();
			objAlt.host.Value(this.alternative_host.Value());
			objAlt.port.value(this.alternative_port.value());
			objAlt.protocol.Value(this.protocol.Value());
			objT.passive_mode.Value(this.alternative_passive_mode.Value());
		}
		else {
			if (this.operation.Value().equalsIgnoreCase("receive")) {
				strDataSourceType = this.protocol.Value();
				if (strDataSourceType.length() <= 0) {
					strDataSourceType = "local";
				}
				changeDirValues4Receive();
				SOSConnection2OptionsAlternate objT = this.getConnectionOptions().Source();
				objT.host.Value(this.host.Value());
				objT.port.value(this.port.value());
				objT.protocol.Value(this.protocol.Value());
				objT.passive_mode.Value(this.passive_mode.Value());
				objT.user = this.user;
				objT.password = this.password;
				objT.ssh_auth_file = this.ssh_auth_file;
				objT.ssh_auth_method = this.ssh_auth_method;
				SOSConnection2OptionsSuperClass objAlt = objT.Alternatives();
				objAlt.host.Value(this.alternative_host.Value());
				objAlt.port.value(this.alternative_port.value());
				objAlt.protocol.Value(this.protocol.Value());
				objT.passive_mode.Value(this.alternative_passive_mode.Value());
			}
			else {
				strDataSourceType = this.getConnectionOptions().Source().protocol.Value();
				if (strDataSourceType.length() <= 0) {
					strDataSourceType = "local";
				}
				changeDirValues();
			}
		}
		logger.debug(String.format("DataSourceType = %1$s", strDataSourceType));
		return strDataSourceType;
	} // private String getDataSourceType

	private void changeDirValues() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::changeDirValues";
		if (this.SourceDir.IsEmpty() == true) {
			this.SourceDir.Value(this.local_dir.Value());
		}
		if (this.TargetDir.IsEmpty() == true) {
			this.TargetDir.Value(this.remote_dir.Value());
		}
	} // private void changeDirValues

	private void changeDirValues4Receive() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::changeDirValues";
		if (this.SourceDir.IsEmpty() == true) {
			this.SourceDir.Value(this.remote_dir.Value());
		}
		if (this.TargetDir.IsEmpty() == true) {
			this.TargetDir.Value(this.local_dir.Value());
		}
	} // private void changeDirValues

	/**
	 * 
	 * \brief DoNotOverwrite
	 * 
	 * \details
	 *
	 * \return boolean
	 *
	 * @return
	 */
	public boolean DoNotOverwrite() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::DoNotOverwrite";
		boolean flgT = this.overwrite_files.value() == false && this.append_files.value() == false;
		return flgT;
	} // private boolean DoNotOverwrite

	public void TransferZeroByteFiles(boolean pzeroByteFiles) {
		this.zeroByteFiles = pzeroByteFiles;
	}

	public boolean TransferZeroByteFiles() {
		return zeroByteFiles;
	}

	public void setZeroByteFilesStrict(boolean pzeroByteFilesStrict) {
		this.zeroByteFilesStrict = pzeroByteFilesStrict;
	}

	public boolean isZeroByteFilesStrict() {
		return zeroByteFilesStrict;
	}

	public void setZeroByteFilesRelaxed(boolean pzeroByteFilesRelaxed) {
		this.zeroByteFilesRelaxed = pzeroByteFilesRelaxed;
	}

	public boolean isZeroByteFilesRelaxed() {
		return zeroByteFilesRelaxed;
	}

	/**
	 * \brief getconnectionOptions
	 *
	 * \details
	 * getter 
	 *
	 * @return the connectionOptions
	 */
	public SOSConnection2Options getConnectionOptions() {
		if (objConnectionOptions == null) {
			objConnectionOptions = new SOSConnection2Options();
		}
		return objConnectionOptions;
	}

	/**
	 * \brief setconnectionOptions - 
	 *
	 * \details
	 * setter 
	 *
	 * @param connectionOptions the value for connectionOptions to set
	 */
	public void setConnectionOptions(SOSConnection2Options connectionOptions) {
		objConnectionOptions = connectionOptions;
	}
}
