/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.event;

import com.sos.scheduler.engine.kernel.*;
import com.sos.scheduler.engine.cplusplus.runtime.annotation.ForCpp;
import java.util.LinkedList;
import java.util.List;
import org.apache.log4j.*;


@ForCpp
public class EventSubsystem extends AbstractHasPlatform
{
    private static final Logger logger = Logger.getLogger(EventSubsystem.class);

    private final List<EventSubscriber> subscribers = new LinkedList<EventSubscriber>();        // Weak references nutzen?
    private int reportEventNesting = 0;


    public EventSubsystem(Platform platform) {
        super(platform);
    }


    public void report(Event e) {
        reportEventNesting++;   // Kann Rekursiv aufgerufen werden.
        try {
            //log().info(this + ": " + e);
            tellSubscribers(e);
        }
        catch (Exception x) { log().error(getClass().getSimpleName() + ".report(" + e.getClass().getName() + "): " + x); }
        finally { reportEventNesting--; }
    }

    
    private void tellSubscribers(Event e) {
        if (reportEventNesting > 1)  throw new SchedulerException("Recursive reportEvent() " + e);
        for (EventSubscriber s: subscribers)
            try { s.onEvent(e); }
            catch (Exception x) { log().error(s + ": " + x.toString()); }
    }


    public void subscribe(EventSubscriber x) { subscribers.add(x); }
    public void unsubscribe(EventSubscriber x) { subscribers.remove(x); }
    
    @Override public String toString() { return getClass().getSimpleName(); }
}
