/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.monitorwindow;

import com.sos.scheduler.engine.kernel.Scheduler;
import com.sos.scheduler.engine.kernel.event.Event;
import com.sos.scheduler.engine.kernel.event.EventSubscriber;
import com.sos.scheduler.engine.kernel.order.jobchain.JobChain;
import com.sos.scheduler.engine.kernel.order.jobchain.Node;
import com.sos.scheduler.engine.kernel.order.Order;
import com.sos.scheduler.engine.kernel.order.OrderStateChangeEvent;
import com.sos.scheduler.engine.kernel.order.jobchain.OrderQueueNode;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;

//TODO Test-Beispiel. Könnte über Annotation aktiviert werden, etwa @Monitor.

/** Beispiel, für die Beobachtung der Scheduler-Ereignisse. */
public class Monitor implements EventSubscriber {
    private final Scheduler scheduler;
    private final Writer output = new PrintWriter(System.err);


    public Monitor(Scheduler scheduler) {
        this.scheduler = scheduler;
    }


    public void close() {
        scheduler.getEventSubsystem().unsubscribe(this);
    }


    public void activate() {
        scheduler.getEventSubsystem().subscribe(this);
        showJobChains();
    }


    @Override public void onEvent(Event e) throws IOException {
        if (e instanceof OrderStateChangeEvent)
            processOrderStateChangeEvent((OrderStateChangeEvent)e);
    }


    private void processOrderStateChangeEvent(OrderStateChangeEvent e) throws IOException {
        Order order = e.getObject();
        JobChain jobChain = order.jobChainOrNull();
        if (jobChain != null) {
            showJobChain(jobChain);
            output.flush();
        }
    }


    private void showJobChains() {
        for (JobChain jobChain: scheduler.getOrderSubsystem().jobChains())  showJobChain(jobChain);
    }


    private void showJobChain(JobChain jobChain) {
        try {
            output.write("***Monitor*** " + jobChain.name() + ": " );
            for (Node node: jobChain.nodes())  showNode(node);
            output.write("\n");
        }
        catch (IOException x) { throw new RuntimeException(x); }
    }


    private void showNode(Node node) throws IOException {
        output.write(node.orderState().string());
        if (node instanceof OrderQueueNode) {
            OrderQueueNode n = (OrderQueueNode)node;
            output.write("(" + n.orderQueue().size() + " orders)");
        }
        output.write("  ");
    }
}
