/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package net.cwroethel.swt.popupcalendar;

import org.eclipse.swt.widgets.Control;

/*******************************************************************************
 * Copyright (c) 2005 Will Roethel. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html Contributors: Will Roethel -
 * initial API and implementation
 ******************************************************************************/

/**
 * Adapter to allow for simple customization of labels used to display the
 * weekdays, the current days of the month, the current day and the remaining
 * days of the previous and next month.
 * 
 * <pre>
 *     PopupCalendar popupCal = new PopupCalendar(shell, style);
 *     ...
 *     popupCal.setDaySelectorStyle( new DaySelectorStyleAdapter() {
 *        public void overrideStyle(Control control, int style) {
 *          switch (style) {
 *            case DaySelectorStyle.WEEKDAYLABEL:
 *              // labels showing the weekday names
 *              break;
 *   
 *            case DaySelectorStyle.BLANKLABEL:
 *              // labels used for the remaining days of the previous and next month
 *              break;
 *   
 *            case DaySelectorStyle.DAYLABEL:
 *              // override the labels for the days of the current month
 *              control.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));
 *              break;
 *   
 *            case DaySelectorStyle.TODAYLABEL:
 *              // override the label showing the current day of the month
 *              break;
 *          }
 *        }
 *     });
 * </pre>
 * 
 * @author W. Roethel, http://www.cwroethel.net
 * @version $Revision: 1.2 $
 */
public abstract class DaySelectorStyleAdapter extends DaySelectorStyle {

    /**
     * Method used to override the default settings.
     * 
     * @param component Control
     * @param style int
     */

    public void overrideStyle(Control component, int style) {
    }
}
