/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/


package sos.scheduler.CheckRunHistory;

import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.SOSOptionHostName;
import com.sos.JSHelper.Options.SOSOptionPortNumber;

/**
 * \class 		JobSchedulerCheckRunHistoryOptions - Check the last job run
 *
 * \brief 
 * An Options as a container for the Options super class. 
 * The Option class will hold all the things, which would be otherwise overwritten at a re-creation
 * of the super-class.
 *
 *

 *
 * see \see J:\E\java\development\com.sos.scheduler\src\sos\scheduler\jobdoc\JobSchedulerCheckRunHistory.xml for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by JobDocu2OptionsClass.xslt from http://www.sos-berlin.com at 20110224143553 
 * \endverbatim
 */
@JSOptionClass(name = "JobSchedulerCheckRunHistoryOptions", description = "Check the last job run")
public class JobSchedulerCheckRunHistoryOptions extends JobSchedulerCheckRunHistoryOptionsSuperClass {
	/**
	 * 
	 */
	private static final long	serialVersionUID	= -3625891732295134070L;
	private final String					conClassName						= "JobSchedulerCheckRunHistoryOptions";  //$NON-NLS-1$
	@SuppressWarnings("unused")
	private static Logger		logger			= Logger.getLogger(JobSchedulerCheckRunHistoryOptions.class);

    /**
    * constructors
    */
    
	public JobSchedulerCheckRunHistoryOptions() {
	} // public JobSchedulerCheckRunHistoryOptions

	public JobSchedulerCheckRunHistoryOptions(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public JobSchedulerCheckRunHistoryOptions

		//

	public JobSchedulerCheckRunHistoryOptions (HashMap <String, String> JSSettings) throws Exception {
		super(JSSettings);
	} // public JobSchedulerCheckRunHistoryOptions (HashMap JSSettings)
/**
 * \brief CheckMandatory - prft alle Muss-Optionen auf Werte
 *
 * \details
 * @throws Exception
 *
 * @throws Exception
 * - wird ausgelst, wenn eine mandatory-Option keinen Wert hat
 */
		@Override  // JobSchedulerCheckRunHistoryOptionsSuperClass
	public void CheckMandatory() {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()
		
		/**
		 * \var PortNumber : The scheduler communication port.
		 * The scheduler communication port.
		 *
		 */ 
		@JSOptionDefinition(name = "PortNumber", description = "The Job Scheduler communication port", key = "PortNumber", type = "SOSOptionPortNumber", mandatory = false)
		public SOSOptionPortNumber	PortNumber		= new SOSOptionPortNumber( // ...
															this, // ....
															conClassName + ".PortNumber", // ...
															"The Job Scheduler communication port", // ...
															"0", // ...
															"4444", // ...
															false);

		/**
		 * \brief getPortNumber : The Job Scheduler communication port.
		 * 
		 * \details
		 * The Job Scheduler communication port.
		 *
		 * \return The Job Scheduler communication port.
		 *
		 */
		public SOSOptionPortNumber getPortNumber() {
			return PortNumber;
		}

		/**
		 * \brief setPortNumber : The Job Scheduler communication port.
		 * 
		 * \details
		 * The Job Scheduler communication port.
		 *
		 * @param SOSOptionPortNumber : The Job Scheduler communication port.
		 */
		public void setPortNumber(SOSOptionPortNumber p_PortNumber) {
			this.PortNumber = p_PortNumber;
		}
		
		/**
		 * \var HostName : The name of the Job Scheduler host.
		 * The name of the Job Scheduler host.
		 *
		 */ 
		@JSOptionDefinition(name = "HostName", description = "The name of the Job Scheduler host", key = "HostName", type = "SOSOptionString", mandatory = false)
		public SOSOptionHostName	HostName	= new SOSOptionHostName( // ...
														this, // ....
														conClassName + ".HostName", // ...
														"The name of the Job Scheduler host", // ...
														"", // ...
														"localhost", // ...
														false);
		/**
		 * \brief getHostName : The name of the Job Scheduler host.
		 * 
		 * \details
		 * The name of the Job Scheduler host.
		 *
		 * \return The name of the Job Scheduler host.
		 *
		 */
		public SOSOptionHostName getHostName() {
			return HostName;
		}

		/**
		 * \brief setHostName : The name of the Job Scheduler host.
		 * 
		 * \details
		 * The name of the Job Scheduler host.
		 *
		 * @param SOSOptionHostName : The name of the Job Scheduler host.
		 */
		public void setHostName(SOSOptionHostName p_HostName) {
			this.HostName = p_HostName;
		}

}

