/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest;


import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author Joacim Zschimmer <j@zsch.de>
 * @since 2004-08-10
 * @version 1.0
 *
 *Arguments berprft die Gltigkeit und Vollstndigkeit der Eingabeparameter
 */
public class Arguments
{
    static class Argument
    {
        Argument(String value )  { _value = value; }
        
        final String      _value;
        boolean           _read = false;
    };
    
    //-----------------------------------------------------------------------------------------
    
    final Map _arguments = new HashMap();

    //--------------------------------------------------------------------------------Arguments
    /**
     * Konstruktor, der die Eingabeparameter als Zeichenkettenfeld erhlt
     */
    Arguments( String[] args )
    throws
        Exception
    {
        for( int i = 0; i < args.length; i++ )
        {
            String arg = args[i];
            if( !arg.startsWith( "-" ) )  throw new Exception( "invalid parameter: " + arg );
     
            int equal = arg.indexOf( "=" );
            if( equal == -1 )  throw new Exception( "invalid parameter: " + arg );
            
            _arguments.put( arg.substring( 0, equal + 1 ), new Argument( arg.substring( equal + 1 ) ) );   
        }
    }

    //--------------------------------------------------------------------------------as_string
    
    String as_string( String option )
    throws
        Exception
    {
        return as_string( option, null );
    }
    
    //--------------------------------------------------------------------------------as_string
    
    String as_string( String option, String default_ )
    throws
        Exception
    {
        Argument argument = (Argument)_arguments.get( option );
        
        if( argument == null )
        {
            if( default_ == null )  throw new Exception( "parameter is missing: " + option );
            return default_;
        }
        
        argument._read = true;
        return argument._value;
    }

    //-----------------------------------------------------------------------------------as_int
    
    int as_int( String option )
    throws
        Exception
    {
        String result = as_string( option );
        
        try
        {
            return Integer.parseInt( result );
        }
        catch( Exception x )
        {
            throw new Exception( "parameter [" + option + "] is invalid: " + x, x );
        }
    }

    //-----------------------------------------------------------------------------------as_int
    
    int as_int( String option, int default_ )
    throws
        Exception
    {
        String result = as_string( option, "" );
        if( result.equals( "" ) )  return default_;
    
        return as_int( option );
    }
		
		//  -------------------------------------------------------------------------as_int_inRange
		
		int as_int_inRange ( String option, int min, int max, int default_ ) throws Exception {
			
			int value = as_int(option, default_);
			
			if ( value==default_ || (value>=min && value<=max) )
				return value;
			else
				throw new Exception( "parameter [" + option + "] not in range " + min + ".." + max );
		}
		
		//---------------------------------------------------------------------------check_all_used
		
    void check_all_used()
    throws
        Exception
    {
        for( Iterator it = _arguments.entrySet().iterator(); it.hasNext(); )
        {
            Map.Entry entry    = (Map.Entry)it.next();
            String    option   = (String)  entry.getKey();
            Argument  argument = (Argument)entry.getValue();
            
            if( !argument._read )  throw new Exception( "unknown parameter: " + option );
        }
    }
}

