/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/

package sos.stresstest;

import java.io.*;
import java.lang.reflect.*;
import java.text.SimpleDateFormat;
import java.util.*;

import sos.connection.SOSConnection;
import sos.settings.SOSProfileSettings;
import sos.util.SOSLogger;
import sos.util.SOSStandardLogger;


/**
 * @author Joacim Zschimmer <j@zsch.de>
 * @since 2004-08-10
 * @version 1.0
 * @author Andreas Pschel <andreas.pueschel@sos-berlin.com>
 * @since 2004-11-21
 * @version 1.1
 * 
 * StresstestThread wird von Stresstest instanziiert. Wie oft hngt von dem Parameter
 * -threads= ab. Jede Instanz vom StresstestThread macht eine Verbindung zum Datenbankserver 
 * auf, die dann von der StresstestSession fr die bermittlung der SQL-Anweisungen 
 * genutzt wird
 */

public class StresstestThread  extends Thread
{
    //----------------------------------------------------------------------------------Time_record
   
    class Time_record
    {
        Time_record( int session_number, String action_name )
        {
            _session_number = session_number;
            _action_name    = action_name;
        }
        
        void stop()
        {
            _elapsed_msec = System.currentTimeMillis() - _elapsed_msec;
        }
        
        int             _start_time;
        int             _session_number;
        String          _action_name;
        long            _elapsed_msec   = System.currentTimeMillis();  // stop() errechnet dann die verstrichene Zeit
        String          _timestamp      = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" ).format( new Date() );
        Throwable       _throwable      = null;
    }
   
    //---------------------------------------------------------------------------------------------
    
    private final int           		_thread_index;
    private final SOSConnection 		_connection;
    private final SOSLogger     		_logger;
    private final SOSProfileSettings    _settings; 
    int                         		_current_session_number = 0;
    final         List          		_time_records           = new LinkedList();
    final         Stresstest    		_stresstest;
    private static boolean              _first = true;
    
       
    //------------------------------------------------------------------------------StresstestThread

    /**Konstruktor 
     * @param stresstest der aufrufende Stresstest bergibt sich selber als Parameter
     * @param thread_index Nummer des threads
     * @throws Exception
     */
    public StresstestThread( Stresstest stresstest, int thread_index )
    throws
        Exception
    {
        Time_record  time_record = new Time_record( 0, "(Thread start)" );

        
        _stresstest   = stresstest;
        _thread_index = thread_index;
 
        _logger       = new SOSStandardLogger( new File( _stresstest._log_dir, thread_index + ".log" ).toString(), Stresstest._verbosity );

        if (_stresstest._configuration_file != null && _stresstest._configuration_file.toString().length() > 0) {
        	_connection   = SOSConnection.createInstance( _stresstest._configuration_file.toString(), _logger );
        	_connection.connect();
        } else {
        	_connection = null;
        }

        if (_stresstest._settings_file != null && _stresstest._settings_file.toString().length() > 0) {
        	_settings  = new SOSProfileSettings(_stresstest._settings_file.getAbsolutePath(), _logger); 
        } else {
        	_settings  = null;
        }
        
        add_time_record( time_record );
    }
   

    //--------------------------------------------------------------------------------getConnection    
    /** Liefert die aktuelle Verbindung zurck
     * @return aktuelle Verbindung
     */    
    public SOSConnection getConnection()  { return _connection; }
    
    /** Liefert die aktuelle Verbindung zurck
     * @return aktuelle Verbindung
     */    
    public SOSConnection get_connection()  { return _connection; }
    

    //------------------------------------------------------------------------------------getLogger    
    /** Liefert den aktuellen Logger zurck
     * @return aktueller Logger
     */    
    public SOSLogger getLogger()  { return _logger; }
    
    /** Liefert den aktuellen Logger zurck
     * @return aktueller Logger
     */    
    public SOSLogger get_logger()  { return _logger; }
    

    /** Gibt den Stresstest zurck, der StresstestThread instanziiert hat
	 * @return Returns the _stresstest.
	 */
	public Stresstest get_stresstest() {
		return _stresstest;
	}

    /** Gibt den Index des Threads zurck
	 * @return Returns the _thread_index.
	 */
	public int get_thread_index() {
		return _thread_index+1;
	}

    /** Gibt das Settings-Objekt zurck
	 * @return Returns the _settings
	 */
	public SOSProfileSettings get_settings() {
		return _settings;
	}


    /**
     * run() Startmethode des Threads, die sich nach Ablauf des Zeitfensters selber
     * beendet
     */
    public void run() throws RuntimeException
    {
        try
        {
            while( System.currentTimeMillis() < _stresstest._until_msec )
            {
                _current_session_number++;
                run_session();
            }
        }
        catch( Exception x )
        {
            CharArrayWriter buffer       = new CharArrayWriter();
            PrintWriter     print_writer = new PrintWriter( buffer ); 
            
            x.printStackTrace( print_writer );
            print_writer.flush();
            
            try
            {
                _logger.error( "[" + _stresstest._test_class.getName() + "]\taborted. " + x + "\n" + buffer );
            }
            catch( Exception xx ) {
            }
            throw new RuntimeException ("Stresstest Thread aborted due to errors: " + x.getMessage());
        }
        finally
        {
            if( _connection != null ) 
            {
                try { _connection.rollback();   } catch( Exception x ) {}
                try { _connection.disconnect(); } catch( Exception x ) {}
            }
        }
    }
    
   //----------------------------------------------------------------------------------run_session    
    /**Start einer Instanz von StresstestSession. 
     * @throws Exception
     */
    
     private void run_session() throws Exception
    {   
    	StresstestSession session = (StresstestSession)_stresstest._test_class_constructor.newInstance( new Object[]{ this });
    	boolean _doInit = false;

        try 
        {
            _logger.debug3( "[" + _stresstest._test_class.getName() + " " + _current_session_number + "]\t[start]" );
            
            synchronized (this) {
            	if (_first) {
            		_first = false;
            		_doInit = true;
            	}
            }
       		
            if (_doInit) session.init();

            
      		session.executeChain();
    		
        }
        catch( Exception xx )
        {
            Throwable x = xx;

            if( x instanceof InvocationTargetException  &&  x.getCause() != null )  x = x.getCause();
            CharArrayWriter buffer       = new CharArrayWriter();
            PrintWriter     print_writer = new PrintWriter( buffer ); 
            
            x.printStackTrace( print_writer );
            print_writer.flush();
            
            _logger.error( "[" + _stresstest._test_class.getName() + " " + _current_session_number + "]\t" + x + "\n" + buffer );
            throw new Exception (x.getMessage());
        }
        finally
        {
            //Time_record  time_record = new Time_record( -1, "(Destruction)" );

            session.destruct();

            //add_time_record( time_record );

            _logger.debug3( "[" + _stresstest._test_class.getName() + " " + _current_session_number + "]\t[end]" );
        }
    }

    //------------------------------------------------------------------------------add_time_record
   
    void add_time_record( Time_record time_record )
    {
        time_record.stop();
        _time_records.add( time_record );
    }
   

    void write_data( PrintStream printer )
    throws
        IOException   
    {
        for( Iterator it = _time_records.iterator(); it.hasNext(); )
        {
            Time_record time_record = (Time_record)it.next();
           
            printer.print( _stresstest._timestamp );
            printer.print( '\t' );
            printer.print( time_record._timestamp );
            printer.print( '\t' );
            printer.print( _thread_index );
            printer.print( '\t' );
            printer.print( time_record._session_number );
            printer.print( '\t' );
            printer.print( time_record._action_name );
            printer.print( '\t' );
            printer.print( time_record._elapsed_msec );
            printer.print( '\t' );
            
            if( time_record._throwable != null )
            	printer.print( time_record._throwable );
            
            printer.print( '\n' );
        }
    }
   
}
