/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: env.cxx 4038 2005-12-15 10:02:25Z jz $

#include "precomp.h"
#include "sos.h"
#include "../file/anyfile.h"
#include "env.h"

#include <set>

namespace sos {

typedef std::set<string> Env;

static bool set_environment_from_sos_ini_executed = false;
static Env  env;

//----------------------------------------------------------------set_environment_from_sos_ini_once

void set_environment_from_sos_ini_once()
{
    Z_MUTEX( hostware_mutex )
    {
        if( !set_environment_from_sos_ini_executed )  set_environment_from_sos_ini();
    }
}

//----------------------------------------------------------------set_environment_from_sos_ini_once

void clear_environment_from_sos_ini()
{
    Z_MUTEX( hostware_mutex )
    {
        if( !set_environment_from_sos_ini_executed )  return;

        set_environment_from_sos_ini_executed = false;

        if( env.size() > 0 )    // Falls bereits destruiert, ist size=0 ...
        {
            for( Env::iterator it = env.begin(); it != env.end(); it++ )
            {
                string n = *it;
                const char* p = strchr( n.c_str(), '=' );

                if( p ) 
                {
                    n.erase( p + 1 - n.c_str() );
                    LOG( "[env] " << n << '\n' );
                    putenv( (char*)n.c_str() );       // Verweis auf String wieder entfernen
                }
            }

            env.clear();
        }
    }
}

//-----------------------------------------------------------------substitute_environment_variables
/*
string substitute_environment_variables( const string& value )
{
    if( !strchr( value.c_str(), '$' ) )  return value;      // Abkrzung


    string result;
    const char* p = value.c_str();

    while( *p )
    {
        if( *p == '$' )
        {
            p++;
            string name;
            while( isalnum( *p )  ||  *p == '_' )  name += *p,  p++;
            result.append( getenv( name.c_str() ) );
        }
        else
            result += *p++;
    }

    return result;
}
*/
//---------------------------------------------------------------------set_environment_from_sos_ini

void set_environment_from_sos_ini()
{
    Z_MUTEX( hostware_mutex )
    {
        set_environment_from_sos_ini_executed = true;

        Any_file section;
        Dynamic_area entry;
        
        section.open( "-in profile -section=env" );

        while( !section.eof() )
        {
            section.get( &entry );

            string name  = section.record_type()->as_string( "entry", entry.byte_ptr() );
            string value = section.record_type()->as_string( "value", entry.byte_ptr() );

            value = z::subst_env( value );
            Env::iterator it = env.insert( name + "=" + value ).first;

            LOG( "[env] " << *it << '\n' );
            putenv( (char*)it->c_str() );
        }
    }
}

} //namespace sos
