/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include <tools.hxx>
#include <string.h>
#include <sos.h>

struct Sos_date : Date 
{
                Sos_date        () : Date(0,0,0);
                Sos_date        ( uint4 date ) : Date(date);
                Sos_date        ( uint day, uint month, uint year ) :
                  Date( day, month, year ) {};

  uint4         date            () const { return GetDate(); };
  void          date            ( uint4 date ) { ChangeDate( date ); };

  uint          year            () const { return GetYear(); };
  uint          month           () const { return GetMonth(); };
  uint          day             () const { return GetDay(); };

  void          year            ( uint y ) { ChangeYear(y); };
  void          month           ( uint m ) { ChangeMonth(m); };
  void          day             ( uint d ) { ChangeDay(d); };

  Bool          valid           () { return IsValid(); };

};


struct Sos_time : Time
{
                Sos_time        () : Time(0,0) {};
                Sos_time        ( uint hour, uint minute ) : Time( hour, minute ) {};

  uint          hour            () { return GetHour(); };
  uint          minute          () { return GetMinute(); };
  void          hour            ( uint h ) { ChangeHour(h); };
  void          minute          ( uint m ) { ChangeMin(m); };

  Bool          valid           ();
};

template <uint size>
struct Sos_fixed_text : Area {
                Sos_fixed_text  ( const char* str = 0 ) : Area(_text,size) { text(str); };          
  operator      const char*     () const { return _text; };
  const char*   text            () const { return _text; };
  void          text            ( const char* str = 0 );
private:
  char _text[size+1];
};

template <uint size>
inline void Sos_fixed_text<size>::Sos_fixed_text( const char* str ) {
  if ( str == 0 ) {
    _text[0] = 0;
    length(0);
    return;
  }
  uint len = min( strlen( str ), size() );
  strncpy( _text, str, len );
  _text[len] = 0;
  length(len);
};

template <uint first, uint last>
struct Sos_number {
                Sos_number  () : _number(first), _nks(0) {}; 
                Sos_number  ( uint4 n, nks = 0 ) : _nks(nks) { number(n); };         
  operator      uint4() const { return _number; };
  uint4         number() { return _number; };
  void          number( uint4 n ) { if (n>=first&&n<=last) _number = n; };
private:
  uint4  _number;
  uint   _nks;
};
