/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: spooler_module_com.cxx 5662 2008-02-25 09:43:28Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com
/*
    Hier sind implementiert

    Com_module_instance_base
    Com_module_instance
    Scripting_engine_module_instance
*/


#include "spooler.h"
#include "../file/anyfile.h"


using namespace std;

namespace sos {
namespace scheduler {

//-------------------------------------------------------------------------------------check_result

bool check_result( const Variant& vt )
{
    if( vt.vt == VT_BOOL     )  return V_BOOL(&vt) != 0;
    if( vt.vt == VT_EMPTY    )  return true;                       // Keine Rckgabe? True, also weiter machen
    if( vt.vt == VT_ERROR  &&  vt.scode == DISP_E_UNKNOWNNAME )  return true;   // Funktion nicht bekannt?
    if( vt.vt == VT_NULL     )  return false;                      // NULL? False
    if( vt.vt == VT_DISPATCH )  return vt.pdispVal != NULL;        // Nothing => False, also Ende

    Variant v = vt;

    HRESULT hr = v.ChangeType( VT_BOOL );
    if( FAILED(hr) )  throw_ole( hr, "VariantChangeType" );

    return V_BOOL(&v) != 0;
}

//-------------------------------------------------------------------Com_module_instance_base::init

void Com_module_instance_base::init()
{
    Module_instance::init();
}

//-------------------------------------------------------------Com_module_instance_base::close__end

void Com_module_instance_base::close__end()
{
    Module_instance::close__end();

    _idispatch = NULL;
}

//------------------------------------------------------------Com_module_instance_base::name_exists

bool Com_module_instance_base::name_exists( const string& name )
{ 
    bool exists;

    if( !_idispatch )  return false;

    map<string,bool>::iterator it = _names.find( name );
    if( it == _names.end() )  exists = _names[name] = com_name_exists( _idispatch, name );
                        else  exists = it->second;

    return exists;                        
}

//-------------------------------------------------------------------Com_module_instance_base::call

Variant Com_module_instance_base::call( const string& name )
{
    In_call in_call ( this, name );

    return com_call( _idispatch, name );
}

//-------------------------------------------------------------------Com_module_instance_base::call

Variant Com_module_instance_base::call( const string& name, const Variant& param, const Variant& param2 )
{
    In_call in_call ( this, name );

    return param2.is_missing()? com_call( _idispatch, name, param )
                              : com_call( _idispatch, name, param, param2 );
}

//-----------------------------------------------------------Com_module_instance_base::property_get
/*
Variant Com_module_instance_base::property_get( const string& name )
{
    return com_property_get( _idispatch, name );
}
*/
//---------------------------------------------------------Com_module_instance::Com_module_instance
#ifdef Z_WINDOWS

Com_module_instance::Com_module_instance( Module* module )
: 
    Com_module_instance_base(module), 
    _zero_(_end_) 
{
    _com_context = new Com_context;
}

//--------------------------------------------------------Com_module_instance::~Com_module_instance

Com_module_instance::~Com_module_instance()
{
    try {
        close__end();   // Synchron
    }
    catch( exception& ) {}
}

//------------------------------------------------------------------------Com_module_instance::init

void Com_module_instance::init()
{
    if( _idispatch )  throw_xc( "Com_module_instance::init" );

    Com_module_instance_base::init();

    HRESULT hr;
    CLSID   clsid = string_as_clsid( _module->_com_class_name );

    if( _module->_filename != "" )
    {
        if( !_module )
        {
            _log.debug( "LoadLibrary " + _module->_filename );
            _com_module = LoadLibrary( _module->_filename.c_str() );
            if( !_com_module )  throw_mswin_error( "LoadLibrary", _module->_filename.c_str() );   
        }


        _DllGetClassObject = (DllGetClassObject_func)GetProcAddress( _com_module, "DllGetClassObject" );
        if( !_DllGetClassObject )  throw_mswin_error( "GetProcAddress DllGetClassObject", _module->_filename.c_str() );


        ptr<IClassFactory> class_factory;
        void*              void_ptr     = NULL;

        hr = _DllGetClassObject( &clsid, (IID*)&IID_IClassFactory, &void_ptr );
        if( FAILED(hr) )  throw_ole( hr, (_module->_filename + "::DllGetClassObject").c_str(), _module->_com_class_name.c_str() );
        class_factory._ptr = static_cast<IClassFactory*>( void_ptr );

        hr = class_factory->CreateInstance( NULL, IID_IDispatch, &void_ptr );
        if( FAILED(hr) )  throw_ole( hr, "CreateInstance", _module->_com_class_name.c_str() );
        _idispatch._ptr = static_cast<IDispatch*>( void_ptr );
    }
    else
    {
        hr = _idispatch.CoCreateInstance( clsid );
        if( FAILED(hr) )  throw_ole( hr, "CoCreateInstance", _module->_com_class_name.c_str() );
    }
}

//------------------------------------------------------------------------Com_module_instance::load

bool Com_module_instance::load()
{
    bool ok = Com_module_instance_base::load();
    if( !ok )  return false;

    if( _com_context  &&  name_exists( "spooler_set_context" ) )
    {
        Variant com_context_vt = +_com_context;
        com_call( _idispatch, "spooler_set_context", &com_context_vt );
    }

    return true;
}

//---------------------------------------------------------------------Com_module_instance::add_obj

void Com_module_instance::add_obj( IDispatch* object, const string& name )
{
    if( name == "spooler_log"    )  _com_context->_log     = object;
    else
    if( name == "spooler"        )  _com_context->_spooler = object;
    else
  //if( name == "spooler_thread" )  _com_context->_thread  = object;
  //else
    if( name == "spooler_job"    )  _com_context->_job     = object;
    else
    if( name == "spooler_task"   )  _com_context->_task    = object;
    else
        throw_xc( "Module_instance::add_obj", name.c_str() );


    Com_module_instance_base::add_obj( object, name );
}

//------------------------------------------------------------------Com_module_instance::close__end

void Com_module_instance::close__end()   // synchron
{
    Com_module_instance_base::close__end();   // Das ist auch synchron

    if( _com_context )  _com_context->close(), _com_context = NULL;

    if( _com_module ) 
    {
        _log.debug( "FreeLibrary " + _module->_filename );
        FreeLibrary( _com_module );
        _com_module = NULL;
    }
}

#endif
//------------------------------Scripting_engine_module_instance::~Scripting_engine_module_instance

Scripting_engine_module_instance::~Scripting_engine_module_instance()
{
    try {
        close__end();  // Synchron
    }
    catch( exception& ) {}
}

//-----------------------------------------------------------Scripting_engine_module_instance::init

void Scripting_engine_module_instance::init()
{
    Com_module_instance_base::init();

    assert( !_script_site );
    _script_site = new Script_site;
    _script_site->_engine_name = _module->_language;
    _script_site->init_engine();
}

//-------------------------------------------------------------Com_module_instance_base::close__end

void Scripting_engine_module_instance::close__end()
{
    close_monitor();

    if( _script_site )
    {
        _idispatch = NULL;

        _script_site->close_engine();
        _script_site = NULL;
    }
}

//------------------------------------------------------------------Scripting_engine_module_instance::add_obj

void Scripting_engine_module_instance::add_obj( IDispatch* object, const string& name )
{
    _script_site->add_obj( object, Bstr(name) );

    Module_instance::add_obj( object, name );
}

//---------------------------------------------------------------------Scripting_engine_module_instance::load

bool Scripting_engine_module_instance::load()
{
    bool ok = Com_module_instance_base::load();
    if( !ok )  return false;

    if( _script_site->_engine_name != _module->_language )  z::throw_xc( "SCHEDULER-117" );

    HRESULT hr = _script_site->_script->SetScriptState( SCRIPTSTATE_INITIALIZED );
    if( FAILED( hr ) )  throw_ole( hr, "IActiveScript::SetScriptState", "SCRIPTSTATE_INITIALIZED" );

    bool is_perl     = string_begins_with( lcase( _script_site->_engine_name ), "perl" );
    bool is_vbscript = lcase( _script_site->_engine_name ) == "vbscript";

    DOM_FOR_EACH_ELEMENT( _module->_text_with_includes.dom_element(), element )
    {
        string text = _module->_text_with_includes.read_text_element( element );
        if( text != "" )
        {
            int line_number = _module->_text_with_includes.text_element_linenr( element );

            if( is_perl )
            {
                text = S() << "\n" 
                              "#line " << line_number << " " 
                                       << quoted_string( _module->_text_with_includes.text_element_filepath( element ), '"', '\\' ) 
                                       << "\n"
                           << text;

                --line_number;
            }
            else
            if( is_vbscript )
                --line_number;

            _script_site->parse( text, line_number );
        }
    }

    _idispatch = _script_site->dispatch();

    return true;
}

//----------------------------------------------------------Scripting_engine_module_instance::start

void Scripting_engine_module_instance::start()
{
#   ifdef Z_WINDOWS
        // init_engine() bereits gerufen
#    else
//        _script_site->init_engine();
#   endif

    HRESULT hr = _script_site->_script->SetScriptState( SCRIPTSTATE_STARTED );
    if( FAILED( hr ) )  throw_ole( hr, "IActiveScript::SetScriptState", "SCRIPTSTATE_STARTED" );
}

//-----------------------------------------------------------Scripting_engine_module_instance::call

Variant Scripting_engine_module_instance::call( const string& name )
{
    SOS_DELETE( _script_site->_script_exception );

    In_call in_call ( this, name );

    try
    {
        return com_call( _idispatch, name );
    }
    catch( const exception& )
    {
        if( _script_site->_script_exception )  throw *_script_site->_script_exception;
                                         else  throw;
    }
}

//-----------------------------------------------------------Scripting_engine_module_instance::call

Variant Scripting_engine_module_instance::call( const string& name, const Variant& param, const Variant& param2 )
{
    SOS_DELETE( _script_site->_script_exception );

    In_call in_call ( this, name );

    try
    {
        return param2.is_missing()? com_call( _idispatch, name, param )
                                  : com_call( _idispatch, name, param, param2 );
    }
    catch( const exception& )
    {
        if( _script_site->_script_exception )  throw *_script_site->_script_exception;
                                         else  throw;
    }
}

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos
