/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: spooler_module_remote.h 5592 2008-02-03 22:24:12Z jz $

#ifndef __SPOOLER_MODULE_REMOTE_H
#define __SPOOLER_MODULE_REMOTE_H

#include "../zschimmer/com_remote.h"

namespace sos {
namespace scheduler {

//---------------------------------------------------------------------Remote_module_instance_proxy

struct Remote_module_instance_proxy : Com_module_instance_base
{

    enum Call_state
    {
        c_none,

        c_begin,            // call__begin()
        c_connect,          //   Mit Server verbinden
        c_create_instance,  
        c_construct,        
        c_call_begin,       //   spooler_open() 

        c_release_begin,    // Wenn Module_monitor.spooler_task_before() false geliefert hat
        c_release,

        c_finished,
    };


    struct Operation : Async_operation
    {
                                Operation                   ( Remote_module_instance_proxy*, Call_state first_state );
                               ~Operation                   ();

        Async_operation*        begin__start                ();
        bool                    begin__end                  ();

        virtual bool            async_finished_             () const;
        virtual bool            async_continue_             ( Continue_flags flags )                    { return _proxy->continue_async_operation( this, flags ); }
      //virtual bool            async_has_error_            ();
      //virtual void            async_check_error_          ();
        virtual string          async_state_text_           () const;

        string                  state_name                  () const;


        Fill_zero              _zero_;
        Remote_module_instance_proxy* _proxy;
        Call_state             _call_state;
        Multi_qi               _multi_qi;
        bool                   _bool_result;
    };



                                Remote_module_instance_proxy( Module* module )                      : Com_module_instance_base(module), _zero_(_end_) {}
                               ~Remote_module_instance_proxy();

    void                        init                        ();
    bool                        load                        ();
    void                        close                       ();
    bool                        kill                        ();
    bool                        is_remote_host              () const                                { return _process && _process->is_remote_host(); }
  
    void                        add_obj                     ( IDispatch*, const string& name );
  //void                        add_log_obj                 ( Com_log*, const string& name );
    bool                        name_exists                 ( const string& name );
    Variant                     call                        ( const string& name );

    bool                        try_to_get_process          ();

            Async_operation*    close__start                ();
            void                close__end                  ();

    virtual Async_operation*    begin__start                ();
    virtual bool                begin__end                  ();

    virtual Async_operation*    end__start                  ( bool success );
    virtual void                end__end                    ();

    virtual Async_operation*    step__start                 ();
    virtual Variant             step__end                   ();

    virtual Async_operation*    call__start                 ( const string& method );
    virtual Variant             call__end                   ();

    virtual Async_operation*    release__start              ();
    virtual void                release__end                ();

    bool                        continue_async_operation    ( Operation*, Async_operation::Continue_flags );
    void                        check_connection_error      ();
    int                         exit_code                   ();
    int                         termination_signal          ();
    File_path                   stdout_path                 ();
    File_path                   stderr_path                 ();
    bool                        try_delete_files            ( Has_log* );
    std::list<File_path>        undeleted_files             ();
    string                      process_name                () const;

    Fill_zero                  _zero_;

    ptr<object_server::Session> _session;
    ptr<object_server::Proxy>   _remote_instance;
    ptr<Async_operation>        _operation;
    bool                        _end_success;               // Fr end__start()
    int                         _exit_code;
    int                         _termination_signal;

    Fill_end                   _end_;
};

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

#endif
